declare filename "SOStom.dsp"; declare name "SOStom"; //import("stdfaust.lib");

// Port from SuperCollider (SC) to Faust of SynthDef \SOStom in 
// <SuperCollider>/examples/demonstrations/DrumSynths.scd
// e.g., /usr/local/share/SuperCollider/examples/demonstrations/DrumSynths.scd
// based on a Sound-on-Sound 'synth secrets' tutorial:
// http://www.soundonsound.com/sos/Mar02/articles/synthsecrets0302.asp

// SOStom -------
// http://www.soundonsound.com/sos/Mar02/articles/synthsecrets0302.asp

//https://github.com/josmithiii/faust-jos/tree/master/percussion

SR = 44100;
PI = 3.1415926535897932385;

gate = checkbox("gate");
trigger = gate > gate';

db2linear(l) = pow(10, l/20.0);

db2linear(l) = pow(10, l/20.0);

smooth(s) = *(1.0 - s) : + ~ *(s);

ampdb  = hslider("gain",-20,-60,40,0.1);
amp = ampdb : smooth(0.999) : db2linear;

freq = hslider("freq", 90, 90, 150, 1);
//freq = 90;
// freq = 128;
// freq = 150;
sustain = 0.4;
drum_timbre = 1.0;
drum_mode_level = 0.25;

decay(n,x) = x - (x>0.0)/n;
release(n) = + ~ decay(n);
envgate(dur,trigger) = trigger : release(int(dur*float(SR))) : >(0.0);

adsr(a,d,s,r,t) = env ~ (_,_) : (!,_) // the 2 'state' signals are fed back
with {
    env (p2,y) =
        (t>0) & (p2|(y>=1)),          // p2 = decay-sustain phase
        (y + p1*u - (p2&(y>s))*v*y - p3*w*y)	// y  = envelop signal
	*((p3==0)|(y>=eps)) // cut off tails to prevent denormals
    with {
		p1 = (p2==0) & (t>0) & (y<1);         // p1 = attack phase
		p3 = (t<=0) & (y>0);                  // p3 = release phase
		// #samples in attack, decay, release, must be >0
		na = SR*a+(a==0.0); nd = SR*d+(d==0.0); nr = SR*r+(r==0.0);
		// correct zero sustain level
		z = s+(s==0.0)*db2linear(-60);
		// attack, decay and (-60dB) release rates
		u = 1/na; v = 1-pow(z, 1/nd); w = 1-1/pow(z*db2linear(60), 1/nr);
		// values below this threshold are considered zero in the release phase
		eps = db2linear(-120);
    };
};

impulse = 1-1';
oscrs(f) = impulse : nlf2(f,1) : _,!; // sine

saw2(freq) = y with { // newer PTR version (stateless - freq can vary at any speed)
  p0 = float(SR)/float(max(1.0e-7,abs(freq))); // period in samples
  t0 = 1.0/p0; // phase increment
  p = ((_<:(-(1)<:_,_),_) <: selector1,selector2) ~(+(t0)):!,_;
  selector1 = select2(<(0)); // for feedback
  selector2 = select2(<(0), (_<:_,(*(1-p0):+(1)):+), _); // for output
  y = 2*p-1;
};

nlf2(f,r,x) = ((_<:_,_),(_<:_,_) : (*(s),*(c),*(c),*(0-s)) :>
              (*(r),+(x))) ~ cross
with {
  th = 2*PI*f/SR;
  c = cos(th);
  s = sin(th);
  cross = _,_ <: !,_,_,!;
};

perc(att,rel,trigger) = adsr(att,0,1.0,rel,envgate(att,trigger));
sinosc0(freq) = oscrs(freq); // SinOsc at phase fi.zero
pmosc(carfreq,modfreq,index) = oscrs(carfreq + (index*modfreq) * oscrs(modfreq));

drum_mode_env = perc(0.005, sustain, trigger);
drum_mode_sin_1 = sinosc0(freq*0.8) * drum_mode_env * 0.5;
drum_mode_sin_2 = sinosc0(freq) * drum_mode_env * 0.5;
drum_mode_pmosc = pmosc(saw2(freq*0.9), freq*0.85, drum_timbre/1.3) * drum_mode_env * 5;
drum_mode_mix = (drum_mode_sin_1 + drum_mode_sin_2 + drum_mode_pmosc) * drum_mode_level;
stick_env = perc(0.005, 0.01, trigger) * 3;
tom_mix = (drum_mode_mix + stick_env) * 2 * amp;

process = tom_mix;