declare filename "granulator1_4.dsp"; declare name "granulator1_4"; import("stdfaust.lib");

// Controls
N = hslider("density", 4, 1, maxN, 1);
gLength = hslider("grain", 0.11, 0.0001, 0.5, 0.0001);
dLength = hslider("length", 4, 0.0001, 8, 0.0001);

//SR = hslider("SR", 4000, 48000, 192000, 1);
//SR = 48000;
//SR = fconstant(int fSamplingFreq, <math.h>);
SR = ma.SR;

maxN = 32;
//delayBufferSize = SR*8;
delayBufferSize = 48000*16; //??? ma.SR does not work... Set high for high sample rates

// Globals
counter = +(1)%delayLength~_; 						// to iterate through the delay line

//Granular synth variables
grainLength = int(SR*gLength);
delayLength = int(SR*dLength);

//Noise Generator

S(1,F) = F;
S(i,F) = F <: S(i-1,F),_ ;
Divide(n,k) = par(i, n, /(k)) ;
random = +(12345) : *(1103515245) ;
RANDMAX = 2^32 - 1 ;
chain(n) = S(n,random) ~ _;
NoiseN(n) = chain(n) : Divide(n,RANDMAX);

noiser = NoiseN(maxN+1);							//multi channel noiser

NoiseChan(n,0) = noiser:>_,par( j, n-1 , !);
NoiseChan(n,i) = noiser:>par( j, i , !) , _, par( j, n-i-1,!);

noise(i) = (NoiseChan(maxN+1,i) + 1) / 2;           //get nth channel of multi-channel noiser
//-------------Noise Generator End

//Sample & Hold
SH(trig,x) = (*(1 - trig) + x * trig) ~ _;

//Grain Positions
grainOffset(i) = int(SH(1-1',int(delayLength*noise(i)))) ;
grainCounterMaster = +(1)%grainLength~_;      		// universal counter for all grains
grainCounter(i) = (grainCounterMaster + grainOffset(i) ) % grainLength;
grainRandomStartPos(i) = int(SH(int(grainCounter(i)/(grainLength-1)),int(delayLength*noise(i))));
grainPosition(i) = grainCounter(i) + grainRandomStartPos(i);

//Delay Line
buffer(write,read,x) = rwtable(delayBufferSize, 0.0, write%delayLength, x, read%delayLength);

//sin wave for windowing
window(i) = sin(2*ma.PI*grainCounter(i)/(grainLength-1));

process = _<: par(i,maxN,buffer(counter, grainPosition(i))*window(i)*(i<N)/N) :> _,_,_,_;