declare filename "rms_dynamical_compression.dsp"; declare name "rms_dynamical_compression"; // dynamical compression based on RMS

ma = library("maths.lib");
si = library ("signals.lib");

// Converts a number of samples to a duration in seconds.
samp2sec = /(ma.SR);

// angular frequency
w(x) = x*2*ma.PI/ma.SR;

// 1-pole lowpass; for stability, CF should be between 0 and Ny 
// (design by Chamberlin)
lp1p(cf, in) = in*a0 : + ~ *(b1)
with {
        a0 = 1-(b1);
        b1 = exp(w(cf)*(-1));
};

// root mean square measurement using lowpass
rms(window, in) = in <: * : lp1p(window) : sqrt;

// dynamical compression based on RMS
dyn_comp_rms(window, curve, in) = in, cucurve : *, rrmmss, cucurve
with {
		cucurve = (1-min(rrmmss, 1) : pow(curve));
		rrmmss = rms(window, in);
};


gain = hslider("gain", 1., 0.0001, 10.0, 0.00001):si.smoo;
curve = hslider("curve", 1.0, 0.00001, 10000.0, 0.00001):si.smoo;
window = hslider("window", 882, 0, 48000, 1) : samp2sec;

process = _ : dyn_comp_rms(window, curve) : _ *gain , _, _;